#include <queue>
#include <SDL.h>
#include <SDL_image.h>
#include <SDL_ttf.h>

int SDL_main(int, char**)
{
	//inicializace SDL - pouze video
	SDL_Init(SDL_INIT_VIDEO);
	IMG_Init(IMG_INIT_PNG);
	TTF_Init();
	//vytvoen okna
	SDL_Window* MainWindow = SDL_CreateWindow("16 a 32 bitova grafika", //jmno okna
		SDL_WINDOWPOS_CENTERED, //okno bude vytvoeno ve stedu obrazovky
		SDL_WINDOWPOS_CENTERED,
		800, 600, NULL); //ka, vka a flagy okna (ty nepotebujeme)


	//vytvoen rendereru
	SDL_Renderer* renderer = SDL_CreateRenderer(MainWindow, -1, NULL);

	//vytvoen 32 bitov surface								      0xF800, 0x7C0, 0x3E, 1);
	SDL_Surface* SurfaceWith32Transition = SDL_CreateRGBSurface(NULL, 256, 256, 32, 0xFF0000, 0x00FF00, 0x0000FF, 0);
	Uint8* pixels = (Uint8*)SurfaceWith32Transition->pixels;
	SDL_PixelFormat* format = SurfaceWith32Transition->format;
	for (int a = 0; a < 256; a++)
		for (int b = 0; b < 256; b++)
		{
			Uint8* pixel = pixels + a*SurfaceWith32Transition->pitch + b*format->BytesPerPixel;
			*(Uint32*)pixel = a << format->Rshift | b << format->Gshift;
		}
	SDL_Texture* TextureWith32Transition = SDL_CreateTextureFromSurface(renderer, SurfaceWith32Transition);
	SDL_Rect* Transition32Rect = new SDL_Rect;
	SDL_FreeSurface(SurfaceWith32Transition);
	Transition32Rect->x = Transition32Rect->y = 0;
	Transition32Rect->w = Transition32Rect->h = 256;


	//vytvoen 16 bitov surface								      0xF800, 0x7C0, 0x3E, 1);
	SDL_Surface* SurfaceWith16Transition = SDL_CreateRGBSurface(NULL, 256, 256, 16, 0xF00, 0x0F0, 0x00F, 0xF000);
	pixels = (Uint8*)SurfaceWith16Transition->pixels;
	format = SurfaceWith16Transition->format;
	for (int a = 0; a < 256; a++)
		for (int b = 0; b < 256; b++)
		{
			int GreenColor = b >> format->Gloss;
			int RedColor = a >> format->Rloss;
			Uint8* pixel = pixels + a*SurfaceWith16Transition->pitch + b*format->BytesPerPixel;
			*(Uint16*)pixel = RedColor << format->Rshift | GreenColor << format->Gshift | 0xF << format->Ashift;
		}
	SDL_Texture* TextureWith16Transition = SDL_CreateTextureFromSurface(renderer, SurfaceWith16Transition);
	SDL_FreeSurface(SurfaceWith16Transition);
	SDL_Rect* Transition16Rect = new SDL_Rect;
	Transition16Rect->x = 0;
	Transition16Rect->y = Transition16Rect->w = Transition16Rect->h = 256;


	//animovana Surface
	SDL_Surface* AnimatedSurface = SDL_CreateRGBSurface(NULL,256,100,32,0xFF0000,0x00FF00,0x0000FF,0);
	pixels = (Uint8*)AnimatedSurface->pixels;
	format = AnimatedSurface->format;
	for (int a = 0; a < 255; a++)
		for (int b = 0; b < 100; b++)
		{
			Uint8* pixel = pixels + b*AnimatedSurface->pitch + a*format->BytesPerPixel;
			*(Uint32*)pixel = a << format->Rshift;
		}
	SDL_Rect* AnimatedRect = new SDL_Rect;
	AnimatedRect->x = 300;
	AnimatedRect->y = 200;
	AnimatedRect->w = 256;
	AnimatedRect->h = 100;


	bool End = false;								//promnn, kter d hlavn smyku
	SDL_Event* event = new SDL_Event;				//struktura, kter uchovv udlosti
	while (!End)									//zatek hlavn smyky
	{
		while (SDL_PollEvent(event))				//naten udlosti z fronty
		{
			if (event->type == SDL_QUIT)				//manipulace s udlost
				End = true;							//ukonen aplikace
		}

		//update prechodu
		std::queue<int> QueueWithColors;
		for (int a = 1; a < 256; a++)
		{
			Uint8* pixel = pixels + a*format->BytesPerPixel;
			QueueWithColors.push(*(Uint32*)pixel);
		}
		QueueWithColors.push(*(Uint32*)pixels);
		for (int a = 0; a < 256; a++)
		{
			int ColorToUse = QueueWithColors.front();
			for (int b = 0; b < 100; b++)
			{
				Uint8* pixel = pixels + b*AnimatedSurface->pitch + a*format->BytesPerPixel;
				*(Uint32*)pixel = ColorToUse;
			}
			QueueWithColors.pop();
		}
		




		SDL_SetRenderDrawColor(renderer, 0, 0, 255, 0);		    //nastaven barvy, kterou se bude vykreslovat pozad
		SDL_RenderClear(renderer);								//vykreslen pozad

		//vykreslen pechodu
		SDL_RenderCopy(renderer, TextureWith32Transition, NULL, Transition32Rect);
		SDL_RenderCopy(renderer, TextureWith16Transition, NULL, Transition16Rect);

		//vykresleni animovaneho prechodu
		SDL_Texture* AnimatedTexture = SDL_CreateTextureFromSurface(renderer, AnimatedSurface);
		SDL_RenderCopy(renderer,AnimatedTexture,NULL, AnimatedRect);
		SDL_DestroyTexture(AnimatedTexture);

		SDL_RenderPresent(renderer);


	}

	//nen poteba mazat jednotliv objekty, po ukonen programu voln msto automaticky promae OS

	TTF_Quit();								//ukonen SDL_ttf
	IMG_Quit();								//ukonen SDL_Image
	SDL_Quit();								//Ukonen SDL
	return 0;
}